(function() {
  var __slice = Array.prototype.slice;
  (function(exports) {
    var DAY, HOUR, MINUTE, ch, dialHours, flag, format, isDate, isLeapYear, isoWeek, locales, monthDayOfYear, pad, padding, paddings, parse, spaced, specifiers, transforms, tz, weekOfYear, _ref;
    locales = {
      en_US: {
        day: {
          abbrev: "Sun Mon Tue Wed Thu Fri Sat".split(/\s/),
          full: "Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(/\s/)
        },
        month: {
          abbrev: "Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(/\s/),
          full: "January February March April May June July August September October November December".split(/\s/)
        },
        dateFormat: "%m/%d/%y",
        timeFormat: "%H:%M:%S",
        dateTimeFormat: "%a %b %_d %H:%M:%S %Y",
        meridiem: ["am", "pm"]
      }
    };
    MINUTE = 1000 * 60;
    HOUR = MINUTE * 60;
    DAY = HOUR * 24;
    pad = function(number, padding, char) {
      var string;
      string = String(number);
      return "" + (new Array((padding - string.length) + 1).join(char)) + string;
    };
    monthDayOfYear = [1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335];
    isLeapYear = function(date) {
      var year;
      year = date.getUTCFullYear();
      if (year % 400 === 0) {
        return true;
      } else if (year % 100 === 0) {
        return false;
      } else if (year % 4 === 0) {
        return true;
      } else {
        return false;
      }
    };
    weekOfYear = function(date, startOfWeek) {
      var day, diff, nyd, remaining, utc, week, weekStart;
      utc = new Date(Date.UTC(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate()));
      nyd = new Date(Date.UTC(date.getUTCFullYear(), 0, 1));
      diff = (utc.getTime() - nyd.getTime()) / DAY;
      day = utc.getUTCDay();
      if (nyd.getUTCDay() === startOfWeek) {
        weekStart = 0;
      } else {
        weekStart = 7 - nyd.getUTCDay() + startOfWeek;
        if (weekStart === 8) {
          weekStart = 1;
        }
      }
      remaining = diff - weekStart;
      week = 0;
      if (diff >= weekStart) {
        week++;
        diff -= weekStart;
        week += Math.floor(diff / 7);
      }
      return week;
    };
    isoWeek = function(date) {
      var ny, nyd, offset, week;
      nyd = new Date(Date.UTC(date.getUTCFullYear(), 0, 1)).getUTCDay();
      offset = nyd > 1 && nyd <= 4 ? 1 : 0;
      week = weekOfYear(date, 1) + offset;
      if (week === 0) {
        ny = new Date(Date.UTC(date.getUTCFullYear() - 1, 0, 1));
        nyd = ny.getUTCDay();
        week = nyd === 4 || (nyd === 3 && isLeapYear(ny)) ? 53 : 52;
        return [week, date.getUTCFullYear() - 1];
      } else if (week === 53 && !(nyd === 4 || (nyd === 3 && isLeapYear(date)))) {
        return [1, date.getUTCFullYear() + 1];
      } else {
        return [week, date.getUTCFullYear()];
      }
    };
    dialHours = function(date) {
      var hours;
      hours = Math.floor(date.getUTCHours() % 12);
      if (hours === 0) {
        return 12;
      } else {
        return hours;
      }
    };
    specifiers = {
      a: function(date, locale) {
        return locale.day.abbrev[date.getUTCDay()];
      },
      A: function(date, locale) {
        return locale.day.full[date.getUTCDay()];
      },
      d: function(date) {
        return date.getUTCDate();
      },
      e: function(date) {
        return date.getUTCDate();
      },
      j: function(date) {
        var days, month;
        month = date.getUTCMonth();
        days = monthDayOfYear[month];
        if (month > 2 && isLeapYear(date)) {
          days++;
        }
        days += date.getUTCDate() - 1;
        return days;
      },
      u: function(date) {
        var day;
        day = date.getUTCDay();
        if (day === 0) {
          day = 7;
        }
        return day;
      },
      w: function(date) {
        return date.getUTCDay();
      },
      U: function(date) {
        return weekOfYear(date, 0);
      },
      W: function(date) {
        return weekOfYear(date, 1);
      },
      V: function(date) {
        var iso;
        return iso = isoWeek(date)[0];
      },
      G: function(date) {
        var iso;
        return iso = isoWeek(date)[1];
      },
      g: function(date) {
        var iso;
        return iso = isoWeek(date)[1] % 100;
      },
      m: function(date) {
        return date.getUTCMonth() + 1;
      },
      h: function(date, locale) {
        return locale.month.abbrev[date.getUTCMonth()];
      },
      b: function(date, locale) {
        return locale.month.abbrev[date.getUTCMonth()];
      },
      B: function(date, locale) {
        return locale.month.full[date.getUTCMonth()];
      },
      y: function(date) {
        return date.getUTCFullYear() % 100;
      },
      Y: function(date) {
        return date.getUTCFullYear();
      },
      C: function(date) {
        return Math.floor(date.getFullYear() / 100);
      },
      D: function(date) {
        return tz("%m/%d/%y", date);
      },
      x: function(date, locale) {
        return tz(locale.dateFormat, date, locale);
      },
      F: function(date) {
        return tz("%Y-%m-%d", date);
      },
      l: function(date) {
        return dialHours(date);
      },
      I: function(date) {
        return dialHours(date);
      },
      k: function(date) {
        return date.getUTCHours();
      },
      H: function(date) {
        return date.getUTCHours();
      },
      P: function(date, locale) {
        return locale.meridiem[Math.floor(date.getUTCHours() / 12)];
      },
      p: function(date, locale) {
        return locale.meridiem[Math.floor(date.getUTCHours() / 12)].toUpperCase();
      },
      M: function(date) {
        return date.getUTCMinutes();
      },
      s: function(date) {
        return Math.floor(date.getTime() / 1000);
      },
      S: function(date) {
        return date.getUTCSeconds();
      },
      N: function(date) {
        return (date.getTime() % 1000) * 1000000;
      },
      r: function(date) {
        return tz("%I:%M:%S %p", date);
      },
      R: function(date) {
        return tz("%H:%M", date);
      },
      T: function(date) {
        return tz("%H:%M:%S", date);
      },
      X: function(date, locale) {
        return tz(locale.timeFormat, date, locale);
      },
      c: function(date, locale) {
        return tz(locale.dateTimeFormat, date, locale);
      }
    };
    padding = {
      d: 2,
      U: 2,
      W: 2,
      V: 2,
      g: 2,
      m: 2,
      j: 3,
      C: 2,
      I: 2,
      H: 2,
      k: 2,
      M: 2,
      S: 2,
      N: 9,
      y: 2
    };
    spaced = {
      k: true
    };
    paddings = {
      "-": function(number) {
        return number;
      }
    };
    _ref = {
      "_": " ",
      "0": "0"
    };
    for (flag in _ref) {
      ch = _ref[flag];
      paddings[flag] = (function(ch) {
        return function(number, padding) {
          return pad(number, padding, ch);
        };
      })(ch);
    }
    transforms = {
      none: function(value) {
        return value;
      },
      "^": function(value) {
        return value.toUpperCase();
      }
    };
    format = function(format, date, locale) {
      var flags, i, match, offset, output, prefix, rest, specifier, transform, value, _ref, _ref2, _ref3;
      locale || (locale = locales.en_US);
      offset = date;
      output = [];
      while (format.length) {
        match = /^(.*?)%([-0_^]?)([aAcdDeFHIjklMNpPsrRSTuwXUWVmhbByYcGgCx])(.*)$/.exec(format);
        if (match) {
          _ref = match.slice(1), prefix = _ref[0], flags = _ref[1], specifier = _ref[2], rest = _ref[3];
          value = specifiers[specifier](offset, locale);
          if (padding[specifier]) {
            flag = spaced[specifier] ? "_" : "0";
            for (i = 0, _ref2 = flags.length; (0 <= _ref2 ? i < _ref2 : i > _ref2); (0 <= _ref2 ? i += 1 : i -= 1)) {
              if (paddings[flags[i]]) {
                flag = flags[i];
              }
            }
            value = paddings[flag](value, padding[specifier]);
          }
          transform = transforms.none;
          for (i = 0, _ref3 = flags.length; (0 <= _ref3 ? i <= _ref3 : i >= _ref3); (0 <= _ref3 ? i += 1 : i -= 1)) {
            transform = transforms[flags[i]] || transform;
          }
          value = transform(value);
          if (prefix != null) {
            output.push(prefix);
          }
          output.push(value);
          format = rest;
        } else if (format.length) {
          output.push(format);
          format = "";
        }
      }
      return output.join("");
    };
    isDate = function(object) {};
    parse = function(pattern) {};
    return exports.tz = exports.timezone = tz = function() {
      var splat;
      splat = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      if (!(splat.length && (splat[0] != null))) {
        return null;
      }
      if (splat[0].getTimezoneOffset && splat[0].setUTCFullYear) {
        return offset.apply(null, splat);
      } else if (splat[0].charCodeAt && splat[0].substring) {
        if (splat.length === 1) {
          return parse.apply(null, splat);
        } else {
          return format.apply(null, splat);
        }
      }
    };
  })((typeof module != "undefined" && module !== null) && (module.exports != null) ? module.exports : this.timezone = {});
}).call(this);
